/* $Id$

This file is part of libmspgl
Copyright © 2007  Mikko Rasa, Mikkosoft Productions
Distributed under the LGPL
*/

#ifndef MSP_GL_VERTEXARRAY_H_
#define MSP_GL_VERTEXARRAY_H_

#include <vector>
#include <msp/core/refptr.h>
#include "types.h"

namespace Msp {
namespace GL {

class VertexArray;
class VertexBuffer;

enum VertexFormat
{
	NODATA=0,
	VERTEX2=1,
	VERTEX3,
	VERTEX4,
	NORMAL3=6,
	TEXCOORD1=8,
	TEXCOORD2,
	TEXCOORD3,
	TEXCOORD4,
	COLOR4_UBYTE=12,
	COLOR3_FLOAT=14,
	COLOR4_FLOAT,
};

inline VertexFormat operator,(VertexFormat a, VertexFormat b) { return VertexFormat((a<<4)|b); }
uint get_stride(VertexFormat);

class VertexArrayBuilder
{
public:
	std::vector<float> &data;

	VertexArrayBuilder(VertexArray &, std::vector<float> &);
	void vertex(float x, float y)                     { vertex(x, y, 0, 1); }
	void vertex(float x, float y, float z)            { vertex(x, y, z, 1); }
	void vertex(float, float, float, float);
	void normal(float x, float y, float z)            { nx=x; ny=y; nz=z; }
	void texcoord(float s)                            { texcoord(s, 0, 0, 1); }
	void texcoord(float s, float t)                   { texcoord(s, t, 0, 1); }
	void texcoord(float s, float t, float r)          { texcoord(s, t, r, 1); }
	void texcoord(float s, float t, float r, float q) { ts=s; tt=t; tr=r; tq=q; }
	void color(ubyte r, ubyte g, ubyte b)             { color(r, g, b, 255); }
	void color(ubyte r, ubyte g, ubyte b, ubyte a)    { color(r/255.f, g/255.f, b/255.f, a/255.f); }
	void color(float r, float g, float b)             { color(r, g, b, 1); }
	void color(float r, float g, float b, float a)    { cr=r; cg=g; cb=b; ca=a; }
	~VertexArrayBuilder();
private:
	VertexArray  &array;
	VertexFormat format;
	uint         stride;

	float cr, cg, cb, ca;  // Color
	float ts, tt, tr, tq;  // TexCoord
	float nx, ny, nz;     // Normal
};

class VertexArray
{
public:
	VertexArray(VertexFormat);
	~VertexArray();

	VertexFormat get_format() const { return format; }
	const std::vector<float> &get_data() const { return data; }
	void         use_vertex_buffer();
	void         use_vertex_buffer(VertexBuffer *);
	void         clear();
	RefPtr<VertexArrayBuilder> modify() { return new VertexArrayBuilder(*this, data); }
	void         apply() const;
	void         update_data();
private:
	VertexFormat format;
	std::vector<float> data;
	uint         stride;
	VertexBuffer *vbuf;
	bool         own_vbuf;

	void set_array(unsigned, unsigned, unsigned) const;

	static unsigned enabled_arrays;
};

} // namespace GL
} // namespace Msp

#endif
